/*
 * Copyright (C) 2016 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.akamai.downloader.sample;

import android.app.Fragment;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.res.AssetManager;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.FragmentManager;

import android.util.JsonReader;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.BaseExpandableListAdapter;
import android.widget.Button;
import android.widget.ExpandableListView;
import android.widget.ExpandableListView.OnChildClickListener;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.TextView;
import android.widget.Toast;

import com.akamai.amp.downloader.listener.PrepareDownloadListener;
import com.akamai.amp.downloader.AmpDownloadService;
import com.akamai.amp.downloader.idea.AmpDownloadManager;
import com.akamai.amp.downloader.listener.AmpDownloaderListener;
import com.akamai.amp.exoplayer2.ParserException;
import com.akamai.amp.exoplayer2.offline.Download;
import com.akamai.amp.exoplayer2.offline.DownloadService;
import com.akamai.amp.exoplayer2.trackselection.DefaultTrackSelector;
import com.akamai.amp.exoplayer2.trackselection.MappingTrackSelector;
import com.akamai.amp.exoplayer2.upstream.DataSource;
import com.akamai.amp.exoplayer2.upstream.DataSourceInputStream;
import com.akamai.amp.exoplayer2.upstream.DataSpec;
import com.akamai.amp.exoplayer2.upstream.DefaultDataSource;
import com.akamai.amp.exoplayer2.util.Assertions;
import com.akamai.amp.exoplayer2.util.Log;
import com.akamai.amp.exoplayer2.util.Util;
import com.akamai.amp.utils.LogManager;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/** An activity for selecting from a list of media samples. */
public class SampleChooserActivity extends AppCompatActivity
        implements AmpDownloaderListener, OnChildClickListener {

  private static final String TAG = "SampleChooserActivity";

  private String[] uris;
  private SampleAdapter sampleAdapter;
  private AmpDownloadManager ampDownloadManager;
  private FrameLayout loadingFragment;
  @Nullable
  private StartDownloadDialogHelper startDownloadDialogHelper;

  private boolean removeProgressAfterCancel = true;

  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.sample_chooser_activity);
    sampleAdapter = new SampleAdapter();
    ExpandableListView sampleListView = findViewById(R.id.sample_list);
    sampleListView.setAdapter(sampleAdapter);
    loadingFragment = findViewById(R.id.loading_container);
    sampleListView.setOnChildClickListener(this);

    Intent intent = getIntent();
    String dataUri = intent.getDataString();
    if (dataUri != null) {
      uris = new String[] {dataUri};
    } else {
      ArrayList<String> uriList = new ArrayList<>();
      AssetManager assetManager = getAssets();
      try {
        for (String asset : assetManager.list("")) {
          if (asset.endsWith(".exolist.json")) {
            uriList.add("asset:///" + asset);
          }
        }
      } catch (IOException e) {
        Toast.makeText(getApplicationContext(), R.string.sample_list_load_error, Toast.LENGTH_LONG)
                .show();
      }
      uris = new String[uriList.size()];
      uriList.toArray(uris);
      Arrays.sort(uris);
    }

    loadSample();

    ampDownloadManager = SampleApplication.ampDownloadManager;
    // Start the download service if it should be running but it's not currently.
    // Starting the service in the foreground causes notification flicker if there is no scheduled
    // action. Starting it in the background throws an exception if the app is in the background too
    // (e.g. if device screen is locked).
    try {
      DownloadService.start(this, AmpDownloadService.class);
    } catch (IllegalStateException e) {
      DownloadService.startForeground(this, AmpDownloadService.class);
    }
    setupLoading();
    loadingFragment.setVisibility(View.INVISIBLE);
  }

  private void setupLoading() {
    Bundle b = new Bundle();
    b.putCharSequence(LoadingFragment.DESCRIPTION_KEY, "Loading Tracks");

    Fragment f = new LoadingFragment();
    f.setArguments(b);

    getFragmentManager()
            .beginTransaction()
            .replace(R.id.loading_container, f)
            .commit();
  }

  @Override
  public boolean onCreateOptionsMenu(Menu menu) {
    MenuInflater inflater = getMenuInflater();
    inflater.inflate(R.menu.sample_chooser_menu, menu);
    return true;
  }

  @Override
  public boolean onOptionsItemSelected(MenuItem item) {
    if (item.getItemId() == R.id.cancel_all){
      ampDownloadManager.cancelAllDownloads(removeProgressAfterCancel);
    } else if (item.getItemId() == R.id.remove_all){
      ampDownloadManager.removeAllDownloads();
    } else if (item.getItemId() == R.id.pause_all){
      ampDownloadManager.pauseAllDownloads();
    } else if (item.getItemId() == R.id.resume_all){
      ampDownloadManager.resumeAllDownloads();
    }
    return true;
  }

  @Override
  public void onStart() {
    super.onStart();
    ampDownloadManager.addListener(this);
    sampleAdapter.notifyDataSetChanged();
  }

  @Override
  public void onStop() {
    ampDownloadManager.removeListener(this);
    super.onStop();
  }

  @Override
  public void onDownloadsChanged(Download asset) {
    sampleAdapter.notifyDataSetChanged();
    LogManager.log(TAG, "State= " + asset.state);
    if(asset.state == Download.STATE_FAILED){
      Toast.makeText(getApplicationContext(), R.string.exo_download_failed, Toast.LENGTH_SHORT)
              .show();
    }
  }

  @Override
  public void onRequestPermissionsResult(
          int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
    if (grantResults.length == 0) {
      // Empty results are triggered if a permission is requested while another request was already
      // pending and can be safely ignored in this case.
      return;
    }
    if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
      loadSample();
    } else {
      Toast.makeText(getApplicationContext(), R.string.sample_list_load_error, Toast.LENGTH_LONG)
              .show();
      finish();
    }
  }

  private void loadSample() {
    Assertions.checkNotNull(uris);

    for (int i = 0; i < uris.length; i++) {
      Uri uri = Uri.parse(uris[i]);
      if (Util.maybeRequestReadExternalStoragePermission(this, uri)) {
        return;
      }
    }

    SampleListLoader loaderTask = new SampleListLoader();
    loaderTask.execute(uris);
  }

  private void onSampleGroups(final List<SampleGroup> groups, boolean sawError) {
    if (sawError) {
      Toast.makeText(getApplicationContext(), R.string.sample_list_load_error, Toast.LENGTH_LONG)
              .show();
    }
    sampleAdapter.setSampleGroups(groups);
  }

  @Override
  public boolean onChildClick(
          ExpandableListView parent, View view, int groupPosition, int childPosition, long id) {
    Sample sample = (Sample) view.getTag();
    Intent intent = new Intent(this, PlayerActivity.class);
    sample.addToIntent(intent);
    startActivity(intent);
    return true;
  }

  private void onSampleDownloadButtonClicked(Sample sample) {
    int downloadUnsupportedStringId = getDownloadUnsupportedStringId(sample);
    if (downloadUnsupportedStringId != 0) {
      Toast.makeText(getApplicationContext(), downloadUnsupportedStringId, Toast.LENGTH_SHORT)
              .show();
    } else {
      loadingFragment.setVisibility(View.VISIBLE);
      Sample.UriSample uriSample = (Sample.UriSample) sample;
      if(ampDownloadManager.isVideoDownloaded(uriSample.uri)){
          ampDownloadManager.deleteDownload(uriSample.uri);
        Toast.makeText(getApplicationContext(), R.string.exo_download_removed, Toast.LENGTH_SHORT)
                .show();
        loadingFragment.setVisibility(View.INVISIBLE);
      } else {
        if (startDownloadDialogHelper != null) {
          startDownloadDialogHelper.release();
        }
        startDownloadDialogHelper =
                new StartDownloadDialogHelper(this, getSupportFragmentManager(), sample.name);
        ampDownloadManager.toggleDownload(this, sample.name, uriSample.uri, uriSample.extension, new PrepareDownloadListener() {
          @Override
          public void onSingleDownloadOptionAvailable() {
            ampDownloadManager.startDownload(sample.name);
            loadingFragment.setVisibility(View.INVISIBLE);
          }

          @Override
          public void onMultipleDownloadOptionsAvailable(MappingTrackSelector.MappedTrackInfo mappedTrackInfo, DefaultTrackSelector.Parameters trackSelectorParameters) {
            startDownloadDialogHelper.displayDownloadDialog(mappedTrackInfo, trackSelectorParameters);
            loadingFragment.setVisibility(View.INVISIBLE);
          }

          @Override
          public void onFailed(IOException e) {
            startDownloadDialogHelper.onError(e);
            loadingFragment.setVisibility(View.INVISIBLE);
          }
        });

      }
    }
  }

  private int getDownloadUnsupportedStringId(Sample sample) {
    Sample.UriSample uriSample = (Sample.UriSample) sample;
    if (uriSample.isLive) {
      return R.string.download_live_unsupported;
    }
    String scheme = uriSample.uri.getScheme();
    if (!("http".equals(scheme) || "https".equals(scheme))) {
      return R.string.download_scheme_unsupported;
    }
    return 0;
  }

  private final class SampleListLoader extends AsyncTask<String, Void, List<SampleGroup>> {

    private boolean sawError;

    @Override
    protected List<SampleGroup> doInBackground(String... uris) {
      List<SampleGroup> result = new ArrayList<>();
      Context context = getApplicationContext();
      String userAgent = Util.getUserAgent(context, "ExoPlayerDemo");
      DataSource dataSource =
              new DefaultDataSource(context, userAgent, /* allowCrossProtocolRedirects= */ false);
      for (String uri : uris) {
        DataSpec dataSpec = new DataSpec(Uri.parse(uri));
        InputStream inputStream = new DataSourceInputStream(dataSource, dataSpec);
        try {
          readSampleGroups(new JsonReader(new InputStreamReader(inputStream, "UTF-8")), result);
        } catch (Exception e) {
          Log.e(TAG, "Error loading sample list: " + uri, e);
          sawError = true;
        } finally {
          Util.closeQuietly(dataSource);
        }
      }
      return result;
    }

    @Override
    protected void onPostExecute(List<SampleGroup> result) {
      onSampleGroups(result, sawError);
    }

    private void readSampleGroups(JsonReader reader, List<SampleGroup> groups) throws IOException {
      reader.beginArray();
      while (reader.hasNext()) {
        readSampleGroup(reader, groups);
      }
      reader.endArray();
    }

    private void readSampleGroup(JsonReader reader, List<SampleGroup> groups) throws IOException {
      String groupName = "";
      ArrayList<Sample> samples = new ArrayList<>();

      reader.beginObject();
      while (reader.hasNext()) {
        String name = reader.nextName();
        switch (name) {
          case "name":
            groupName = reader.nextString();
            break;
          case "samples":
            reader.beginArray();
            while (reader.hasNext()) {
              samples.add(readEntry(reader, false));
            }
            reader.endArray();
            break;
          case "_comment":
            reader.nextString(); // Ignore.
            break;
          default:
            throw new ParserException("Unsupported name: " + name);
        }
      }
      reader.endObject();

      SampleGroup group = getGroup(groupName, groups);
      group.samples.addAll(samples);
    }

    private Sample readEntry(JsonReader reader, boolean insidePlaylist) throws IOException {
      String sampleName = null;
      Uri uri = null;
      String extension = null;
      boolean isLive = false;
      String sphericalStereoMode = null;
      Uri subtitleUri = null;
      String subtitleMimeType = null;
      String subtitleLanguage = null;

      reader.beginObject();
      while (reader.hasNext()) {
        String name = reader.nextName();
        switch (name) {
          case "name":
            sampleName = reader.nextString();
            break;
          case "uri":
            uri = Uri.parse(reader.nextString());
            break;
          case "extension":
            extension = reader.nextString();
            break;
          case "is_live":
            isLive = reader.nextBoolean();
            break;
          case "spherical_stereo_mode":
            Assertions.checkState(
                    !insidePlaylist, "Invalid attribute on nested item: spherical_stereo_mode");
            sphericalStereoMode = reader.nextString();
            break;
          case "subtitle_uri":
            subtitleUri = Uri.parse(reader.nextString());
            break;
          case "subtitle_mime_type":
            subtitleMimeType = reader.nextString();
            break;
          case "subtitle_language":
            subtitleLanguage = reader.nextString();
            break;
          default:
            throw new ParserException("Unsupported attribute name: " + name);
        }
      }
      reader.endObject();
      Sample.SubtitleInfo subtitleInfo =
              subtitleUri == null
                      ? null
                      : new Sample.SubtitleInfo(
                      subtitleUri,
                      Assertions.checkNotNull(
                              subtitleMimeType, "subtitle_mime_type is required if subtitle_uri is set."),
                      subtitleLanguage);
     return new Sample.UriSample(
                sampleName,
                uri,
                extension,
                isLive,
                sphericalStereoMode,
                subtitleInfo);
    }

    private SampleGroup getGroup(String groupName, List<SampleGroup> groups) {
      for (int i = 0; i < groups.size(); i++) {
        if (Util.areEqual(groupName, groups.get(i).title)) {
          return groups.get(i);
        }
      }
      SampleGroup group = new SampleGroup(groupName);
      groups.add(group);
      return group;
    }

  }

  private void cancelSingleDownload(Sample sample){
    Sample.UriSample uriSample = (Sample.UriSample) sample;
    ampDownloadManager.cancelDownload(uriSample.uri, removeProgressAfterCancel);
    Toast.makeText(getApplicationContext(), R.string.cancel_confirmation, Toast.LENGTH_LONG)
            .show();
  }

  private OnClickListener downloadOnClickListener = (View view) -> onSampleDownloadButtonClicked((Sample) view.getTag());

  private OnClickListener cancelOnClickListener = (View view) -> cancelSingleDownload((Sample) view.getTag());

  private final class SampleAdapter extends BaseExpandableListAdapter {

    private List<SampleGroup> sampleGroups;

    public SampleAdapter() {
      sampleGroups = Collections.emptyList();
    }

    public void setSampleGroups(List<SampleGroup> sampleGroups) {
      this.sampleGroups = sampleGroups;
      notifyDataSetChanged();
    }

    @Override
    public Sample getChild(int groupPosition, int childPosition) {
      return getGroup(groupPosition).samples.get(childPosition);
    }

    @Override
    public long getChildId(int groupPosition, int childPosition) {
      return childPosition;
    }

    @Override
    public View getChildView(int groupPosition, int childPosition, boolean isLastChild,
                             View convertView, ViewGroup parent) {
      View view = convertView;
      if (view == null) {
        view = getLayoutInflater().inflate(R.layout.sample_list_item, parent, false);
        View downloadButton = view.findViewById(R.id.download_button);
        View cancelButton = view.findViewById(R.id.cancel_button);
        downloadButton.setOnClickListener(downloadOnClickListener);
        cancelButton.setOnClickListener(cancelOnClickListener);
        downloadButton.setFocusable(false);
        cancelButton.setFocusable(false);
      }
      initializeChildView(view, getChild(groupPosition, childPosition));
      return view;
    }

    @Override
    public int getChildrenCount(int groupPosition) {
      return getGroup(groupPosition).samples.size();
    }

    @Override
    public SampleGroup getGroup(int groupPosition) {
      return sampleGroups.get(groupPosition);
    }

    @Override
    public long getGroupId(int groupPosition) {
      return groupPosition;
    }

    @Override
    public View getGroupView(int groupPosition, boolean isExpanded, View convertView,
                             ViewGroup parent) {
      View view = convertView;
      if (view == null) {
        view =
                getLayoutInflater()
                        .inflate(android.R.layout.simple_expandable_list_item_1, parent, false);
      }
      ((TextView) view).setText(getGroup(groupPosition).title);
      return view;
    }

    @Override
    public int getGroupCount() {
      return sampleGroups.size();
    }

    @Override
    public boolean hasStableIds() {
      return false;
    }

    @Override
    public boolean isChildSelectable(int groupPosition, int childPosition) {
      return true;
    }

    private void initializeChildView(View view, Sample sample) {
      view.setTag(sample);
      TextView sampleTitle = view.findViewById(R.id.sample_title);
      sampleTitle.setText(sample.name);

      boolean canDownload = getDownloadUnsupportedStringId(sample) == 0;
      boolean isDownloaded = canDownload && ampDownloadManager.isVideoDownloaded(((Sample.UriSample) sample).uri);
      ImageButton downloadButton = view.findViewById(R.id.download_button);
      downloadButton.setTag(sample);
      downloadButton.setColorFilter(
              canDownload ? (isDownloaded ? 0xFF42A5F5 : 0xFFBDBDBD) : 0xFF666666);
      downloadButton.setImageResource(
              isDownloaded ? R.drawable.ic_download_done : R.drawable.ic_download);
      Button cancelButton = view.findViewById(R.id.cancel_button);
      cancelButton.setTag(sample);
    }
  }

  private static final class SampleGroup {

    public final String title;
    public final List<Sample> samples;

    public SampleGroup(String title) {
      this.title = title;
      this.samples = new ArrayList<>();
    }
  }

  private final class StartDownloadDialogHelper
          implements  DialogInterface.OnClickListener,
          DialogInterface.OnDismissListener {

    private final FragmentManager fragmentManager;

    private TrackSelectionDialog trackSelectionDialog;
    private Context context;
    private String name;

    public StartDownloadDialogHelper(Context context,
            FragmentManager fragmentManager, String name) {
      this.context = context;
      this.fragmentManager = fragmentManager;
      this.name = name;
    }

    public void release() {
      if (trackSelectionDialog != null) {
        trackSelectionDialog.dismiss();
      }
    }

    public void onError(IOException e) {
      Toast.makeText(context, R.string.download_start_error, Toast.LENGTH_LONG)
              .show();
      android.util.Log.e(TAG, "Failed to start download", e);
    }

    public void displayDownloadDialog(MappingTrackSelector.MappedTrackInfo mappedTrackInfo,
                                      DefaultTrackSelector.Parameters trackSelectorParameters){
      trackSelectionDialog =
              TrackSelectionDialog.createForMappedTrackInfoAndParameters(
                      /* titleId= */ com.akamai.amp.downloader.R.string.exo_download_description,
                      mappedTrackInfo,
                      trackSelectorParameters,
                      /* allowAdaptiveSelections =*/ false,
                      /* allowMultipleOverrides= */ true,
                      /* onClickListener= */ this,
                      /* onDismissListener= */ this);
      trackSelectionDialog.show(fragmentManager, /* tag= */ null);
    }

    // DialogInterface.OnClickListener implementation.

    @Override
    public void onClick(DialogInterface dialog, int which) {
      ampDownloadManager.extractSelectedRenderers(trackSelectionDialog);
      ampDownloadManager.startDownload(name);
    }

    // DialogInterface.OnDismissListener implementation.

    @Override
    public void onDismiss(DialogInterface dialogInterface) {
      trackSelectionDialog = null;
      ampDownloadManager.cancelToggle();
    }
  }
}
